"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _signer, _proxyFactoryAddress, _safeSingletonAddress;
Object.defineProperty(exports, "__esModule", { value: true });
const ethers_1 = require("ethers");
const GnosisSafeProxyFactory_json_1 = __importDefault(require("@gnosis.pm/safe-contracts/build/contracts/GnosisSafeProxyFactory.json"));
const GnosisSafe_json_1 = __importDefault(require("@gnosis.pm/safe-contracts/build/contracts/GnosisSafe.json"));
const constants_1 = require("./utils/constants");
const contracts_1 = require("./utils/contracts");
const EthersSafe_1 = __importDefault(require("./EthersSafe"));
class EthersSafeFactory {
    constructor(signer, proxyFactoryAddress, safeSingletonAddress) {
        _signer.set(this, void 0);
        _proxyFactoryAddress.set(this, void 0);
        _safeSingletonAddress.set(this, void 0);
        __classPrivateFieldSet(this, _signer, signer);
        __classPrivateFieldSet(this, _proxyFactoryAddress, proxyFactoryAddress);
        __classPrivateFieldSet(this, _safeSingletonAddress, safeSingletonAddress);
        if (!__classPrivateFieldGet(this, _signer).provider)
            throw new Error('Signer must be connected to a provider');
        this.validateIsDeployed = contracts_1.validateIsDeployedFactory(__classPrivateFieldGet(this, _signer).provider);
    }
    async createSafe(safeAccountConfiguration, deploymentOptions) {
        await this.validateIsDeployed(__classPrivateFieldGet(this, _proxyFactoryAddress), 'ProxyFactory');
        await this.validateIsDeployed(__classPrivateFieldGet(this, _safeSingletonAddress), 'SafeSingleton');
        if (safeAccountConfiguration.owners.length <= 0)
            throw new Error('Invalid owners: it must have at least one');
        if (safeAccountConfiguration.threshold <= 0)
            throw new Error('Invalid threshold: it must be greater than or equal to 0');
        if (safeAccountConfiguration.threshold > safeAccountConfiguration.owners.length)
            throw new Error('Invalid threshold: it must be lower than or equal to owners length');
        const { owners, threshold, to = constants_1.ZERO_ADDRESS, data = constants_1.EMPTY_DATA, fallbackHandler = constants_1.ZERO_ADDRESS, paymentToken = constants_1.ZERO_ADDRESS, payment = 0, paymentReceiver = constants_1.ZERO_ADDRESS } = safeAccountConfiguration;
        const gnosisSafe = await this.deployProxy(deploymentOptions);
        await gnosisSafe
            .setup(owners, threshold, to, data, fallbackHandler, paymentToken, payment, paymentReceiver)
            .then((tx) => tx.wait());
        return await EthersSafe_1.default.create(ethers_1.ethers, gnosisSafe.address, __classPrivateFieldGet(this, _signer));
    }
    async createDeployProxyTransaction(deploymentOptions = {}) {
        const proxyFactory = new ethers_1.Contract(__classPrivateFieldGet(this, _proxyFactoryAddress), GnosisSafeProxyFactory_json_1.default.abi, __classPrivateFieldGet(this, _signer));
        const { data = constants_1.EMPTY_DATA, nonce, callbackAddress } = deploymentOptions;
        if (callbackAddress && nonce) {
            return await proxyFactory.createProxyWithCallback(__classPrivateFieldGet(this, _safeSingletonAddress), data, nonce, callbackAddress);
        }
        else if (nonce) {
            return await proxyFactory.createProxyWithNonce(__classPrivateFieldGet(this, _safeSingletonAddress), data, nonce);
        }
        else {
            return await proxyFactory.createProxy(__classPrivateFieldGet(this, _safeSingletonAddress), data);
        }
    }
    async deployProxy(deploymentOptions = {}) {
        var _a;
        const receipt = await this.createDeployProxyTransaction(deploymentOptions).then((tx) => tx.wait());
        const proxyAddress = (_a = receipt.events) === null || _a === void 0 ? void 0 : _a.find((e) => e.event === 'ProxyCreation').args[0];
        return new ethers_1.Contract(proxyAddress, GnosisSafe_json_1.default.abi, __classPrivateFieldGet(this, _signer));
    }
}
_signer = new WeakMap(), _proxyFactoryAddress = new WeakMap(), _safeSingletonAddress = new WeakMap();
exports.default = EthersSafeFactory;
//# sourceMappingURL=EthersSafeFactory.js.map